TILESIZE = 32 -- dimensions of a cell (and tile also)

QUAD_WIDTH = 5
QUAD_HEIGHT = 5

ENEMIES_PER_QUAD = 4
ROCKS_PER_QUAD = 2

-- generate quad data
local quads = {}
for q = 1, 4 do
  local quad = {}
  for r = 1, QUAD_HEIGHT * QUAD_WIDTH do
    quad[#quad + 1] = 0 -- generate empty cell
  end
  quads[#quads + 1] = quad -- store quad
end

-- return a table containing which cells in a quad are empty
local function getEmptyCells(quad)
  local cells = {}
  for i = 1, #quad do
    if quad[i] == 0 then -- cell is empty so can accomodate an object
      cells[#cells + 1] = i -- add to list of empty cells
    end
  end
  return cells
end

-- generate enemies
for q = 1, 4 do
  local floor, random = math.floor, math.random
  print("generating objects in quad " .. q)
  local quad = quads[q] -- get next quad
  for i = 1, ENEMIES_PER_QUAD do
    local cells = getEmptyCells(quad) -- get empty cells in quad
    local cell = cells[random(1, #cells)] -- choose empty cell at random
    quad[cell] = 1 -- mark cell occupied by enemy
    print("rock created at " .. cell % QUAD_WIDTH .. ", " .. floor(cell / QUAD_WIDTH))
  end
end

-- generate rocks
for q = 1, 4 do
  local floor, random = math.floor, math.random
  local quad = quads[q]
  for i = 1, ROCKS_PER_QUAD do
    local cells = getEmptyCells(quad)
    local cell = cells[random(1, #cells)]
    quad[cell] = 2 -- mark cell occupied by rock
    print("rock created at " .. cell % QUAD_WIDTH .. ", " .. floor(cell / QUAD_WIDTH))
  end
end

-- draw everything to the stage
local canvas = RenderTarget.new(TILESIZE * QUAD_WIDTH * 2, TILESIZE * QUAD_HEIGHT * 2)

local dirt = Bitmap.new( Texture.new("dirt.png", true) ) -- brushes for drawing on canvas
local enemy = Bitmap.new( Texture.new("enemy.png", true) )
local rock = Bitmap.new( Texture.new("rock.png", true) )

local quadPositions = { -- x, y offsets on canvas where we will draw our brushes
  {x = 0, y = 0},
  {x = TILESIZE * QUAD_WIDTH, y = 0},
  {x = 0, y = TILESIZE * QUAD_HEIGHT},
  {x = TILESIZE * QUAD_HEIGHT, y = TILESIZE * QUAD_WIDTH},
}

for i = 1, #quads do
  local p = quadPositions[i]
  local quad = quads[i]

  for r = 0, QUAD_HEIGHT - 1 do
    local brush
    for c = 0, QUAD_WIDTH - 1 do
      local cell = quad[(r * QUAD_WIDTH) + c + 1] -- get next cell
      if cell == 0 then -- determine which brush to draw
        brush = dirt
      elseif cell == 1 then
        brush = enemy
      else
        brush = rock
      end
      brush:setPosition((c * TILESIZE) + p.x, (r * TILESIZE) + p.y) -- set position and draw brush
      canvas:draw(brush)
    end
  end
end

local display = Bitmap.new(canvas) -- show our world
stage:addChild(display)

local p1 = Pixel.new(0x000000, 1, TILESIZE * QUAD_WIDTH * 2, 2) -- horizontal quad divider
p1:setPosition(0, TILESIZE * QUAD_HEIGHT - 1)
stage:addChild(p1)

local p2 = Pixel.new(0x000000, 1, 2, TILESIZE * QUAD_HEIGHT * 2) -- vertical quad divider
p2:setPosition(TILESIZE * QUAD_WIDTH - 1, 0)
stage:addChild(p2)
